﻿/*
Developed by AQU.
This javascript functions toolbox provides helper functions to help :
- consume portal web API : https://docs.microsoft.com/en-us/power-apps/maker/portals/web-api-overview
- Mimic model-driven app Business rules (Show/Hide, Set mandatory/optional, Lock/Unlock )
- Perform common html tasks (such as converting a lookup field into a classic dropdown list)

This toolbox must be made available in the entire portal website. To do so, we reference it as a web file and add the following html node in the  web template "header" of the portal website :
<script content-type="text/javascript" src="/uge_portal_toolbox.js"></script>
web file in DEV is here : https://eaca-candidatures-dev.crm17.dynamics.com/main.aspx?appid=3cc61040-23a9-ec11-9841-00224861970d&pagetype=entityrecord&etn=adx_webfile&id=28c50192-b4b4-ec11-983f-00224860d259

IMPORTANT : As this toolbox is located in header section of the website, when using this toolbox from a modal dialog you must call function with window.parent.<FunctionName>. 
NB: If a window does not have a parent, its parent property is a reference to itself.

The `{{snippets['XXX/XXX/XXX']}}` should be sent as a parameter to the function (ex : messageSnippet) while in the portal toolbox context

    05.04.2022/AQU ==> v1.0
*/


//API Ajax wrapper by Microsoft
(function (webapi, $) {
    function safeAjax(ajaxOptions) {
        var deferredAjax = $.Deferred();

        shell.getTokenDeferred().done(function (token) {
            // add headers for AJAX
            if (!ajaxOptions.headers) {
                $.extend(ajaxOptions, {
                    headers: {
                        "__RequestVerificationToken": token
                    }
                });
            } else {
                ajaxOptions.headers["__RequestVerificationToken"] = token;
            }
            $.ajax(ajaxOptions)
                .done(function (data, textStatus, jqXHR) {
                    validateLoginSession(data, textStatus, jqXHR, deferredAjax.resolve);
                }).fail(deferredAjax.reject); //AJAX
        }).fail(function () {
            deferredAjax.rejectWith(this, arguments); // on token failure pass the token AJAX and args
        });

        return deferredAjax.promise();
    }
    webapi.safeAjax = safeAjax;
})(window.webapi = window.webapi || {}, jQuery)

function TestPortalToolbox() {
    alert("Portal Toolbox is active.");
}

function ConvertLookupToDropdown(entityName, entitySet, lookupAttribute, labelAttribute, currentFrame, additionalQueryOption = "", specialFunction = null, emptyLabel = "") {
    //SetFieldDisplay(lookupAttribute, "hide", currentFrame, false);
    var primaryKeyAttribute = entityName + "id"
    var urlLookupDropdown = "/_api/" + entitySet.toLowerCase() + "?$select=" + primaryKeyAttribute + "," + labelAttribute + additionalQueryOption;
    webapi.safeAjax({
        type: "GET",
        url: urlLookupDropdown,
        contentType: "application/json",
        success: function (res, status, xhr) {
            //console.log(res);
            var selectElement = document.createElement("select");
            var selectElementId = "lookupDropdown_" + lookupAttribute;
            selectElement.setAttribute("name", selectElementId);
            selectElement.setAttribute("id", selectElementId);
            selectElement.setAttribute("class", "form-control picklist");
            selectElement.setAttribute("required", "false");
            selectElement.setAttribute("title", "Sélectionnez un élément dans cette liste");
            selectElement.setAttribute("aria-invalid", "false");
            var emptyOptionElement = document.createElement("option");
            emptyOptionElement.setAttribute("value", "");
            emptyOptionElement.setAttribute("label", "");
            emptyOptionElement.setAttribute("aria-label", "Vide");
            var selectedValue = currentFrame.find("#" + lookupAttribute).first().val();
            selectElement.appendChild(emptyOptionElement);


            if (specialFunction === null) {
                res.value.forEach(function (item) {
                    var optionElement = document.createElement("option");
                    optionElement.setAttribute("value", item[primaryKeyAttribute]);
                    optionElement.innerHTML = item[labelAttribute];
                    if (item[primaryKeyAttribute] === selectedValue) {
                        optionElement.setAttribute("selected", "selected");
                    }
                    selectElement.appendChild(optionElement);
                });
            }
            else {
                res.value.forEach(function (item) { specialFunction(item, selectElement, selectedValue, currentFrame) });
            }

            selectElement.addEventListener("change", function () {
                let lookupIdElement = currentFrame.find("#" + lookupAttribute).get(0);
                lookupIdElement.setAttribute("value", currentFrame.find("#" + selectElementId).first().val());
                lookupIdElement.setAttribute("class", "dirty");
                currentFrame.find("#" + lookupAttribute + "_entityname").first().val(entityName);
                currentFrame.find(lookupIdElement).trigger("change");
            });

            //handle disabled field
            if (currentFrame.find("#" + lookupAttribute).get(0).hasAttribute("disabled")) {//if the lookup field is disabled
                $(selectElement).prop("disabled", true);// we also disable new select element
            }
            //handle preivous dropdown (if any) with same id.
            currentFrame.find("#" + selectElementId).remove();//we clear if there is an existing dropdown with same id in the currentFrame.

            //insert element
            //SetFieldDisplay(lookupAttribute, "show", currentFrame);
            let sibling = currentFrame.find("#" + lookupAttribute + "_name").get(0);
            let parentDiv = sibling.parentNode;
            parentDiv.insertBefore(selectElement, sibling);
            sibling.style.display = 'none';
            //hide lookup icon button :
            var nextElementSibling = currentFrame.find("#" + lookupAttribute + "_entityname").get(0).nextElementSibling;
            if (nextElementSibling !== null) {
                currentFrame.find("#" + lookupAttribute + "_entityname").get(0).nextElementSibling.style.display = 'none';//hide lookup icon button
            }

            //emptyOption
            if (emptyLabel !== "") {
                SetEmptyLabelInDropDown(selectElementId, currentFrame, emptyLabel, emptyLabel);
            }
        }
    });
}

function GetCurrentFrame(iframeId = null) {
    if (iframeId === null) {
        return $(document).contents();
    }
    else {
        return $("#" + iframeId).first().contents();
    }
}

function SetFieldDisplay(attribute, displayStatus, currentFrame, eraseAttributeWhenHiding = true) {
    if (displayStatus === "show") {
        currentFrame.find("#" + attribute).first().closest("td").show();
    }
    else if (displayStatus === "hide") {
        currentFrame.find("#" + attribute).first().closest("td").hide();
        //traitement de l'effacement des données
        if (eraseAttributeWhenHiding) {
            ClearField(attribute, currentFrame);
        }
    }
}

function ClearField(attribute, currentFrame) {
    //console.log("ClearField " + attribute);
    var input = currentFrame.find("#" + attribute);
    if (input.length > 0) {
        var inputType = input.attr('type');

        //radio
        if (input.hasClass("picklist") && input.length > 0 && input.get(0).tagName.toLowerCase() !== "select") {
            //console.log("radio");
            input.find("input[type='radio']:checked").prop('checked', false);
            input.addClass("dirty");
        }
        //checkbox test OK
        else if (typeof inputType !== 'undefined' && inputType !== false && inputType === 'checkbox') {
            //console.log("checkbox");
            input.prop("checked", false)
            input.addClass("dirty");
        }
        //lookup test OK
        else if (typeof inputType !== 'undefined' && inputType !== false && inputType === 'hidden' && currentFrame.find('#' + attribute + "_entityname").length > 0) {
            //console.log("lookup");
            input.parent().find("button.clearlookupfield").click();
            currentFrame.find("#lookupDropdown_" + attribute).val("");//gere le cas des lookup dropdown !!!
        }
        //select|dropdown test OK
        else if (input.length > 0 && input.get(0).tagName.toLowerCase() === "select") {
            //console.log("select|dropdown");
            input.val("");
            input.change();
        }
        //text, integer, date test OK
        else {
            //console.log("text, integer");
            input.val(null);
            input.change();
            if (input.hasClass('datetime')) {
                currentFrame.find("#" + attribute + "_datepicker_description").val(null);
            }
        }
    }
}

function SetFieldInstruction(attribute, instructions, currentFrame, htmlClass = "description", closestTarget = ".control") {
    currentFrame.find("#" + attribute + "_instructions").remove();//we remove previous instruction if any.
    var descriptionDiv = document.createElement("div");
    descriptionDiv.setAttribute("class", htmlClass + " above");
    descriptionDiv.setAttribute("id", attribute + "_instructions");
    descriptionDiv.innerHTML = instructions;
    input = currentFrame.find("#" + attribute).first();
    if (input.attr('type') === "checkbox") {
        input.closest("td").prepend(descriptionDiv);
    }
    else {
        input.closest(closestTarget).prepend(descriptionDiv);
    }
}

function SetFieldAlert(attribute, message, currentFrame, level = "danger") {//success|info|warning|danger
    if (currentFrame.find("#" + attribute).closest("td").find(".alert.alert-" + level).length === 0) {//we SetFieldAlert only if not already present
        var errorDiv = document.createElement("div");
        errorDiv.setAttribute("class", "alert alert-" + level);
        errorDiv.setAttribute("role", "alert");
        errorDiv.innerHTML = message;
        var input = currentFrame.find("#" + attribute).first();
        if (input.attr('type') === "checkbox") {
            input.closest("td").append(errorDiv);
        }
        else {
            input.closest(".control").append(errorDiv);
        }
    }
}

function RemoveFieldAlert(attribute, currentFrame, level = "danger") {
    currentFrame.find("#" + attribute).closest("td").find(".alert.alert-" + level).remove();
}

function GetRadioButtonValue(attribute, currentFrame) {
    return parseInt(currentFrame.find("input[id^='" + attribute + "']:checked").first().val());
}

function SetRadioButtonValue(attribute, value, currentFrame) {
    currentFrame.find("#" + attribute).children('[value="' + value + '"]').prop("checked", true);
    currentFrame.find("#" + attribute).trigger("change");
}

function GetDropdownValue(attribute, currentFrame) {
    const parsed = parseInt(currentFrame.find('#' + attribute).first().find(":selected").val());
    if (isNaN(parsed)) { return -1; }
    return parsed;
}

function CountEmptyRequiredField() {
    var countAllRequired = 0;
    var countEmptyRequired = 0;
    var message = "";
    var select = "$select=";
    var currentValue = null;
    var currentAttributeId = null;

    document.querySelectorAll("[id^=RequiredFieldValidator]").forEach(function (element, index, array) {
        countAllRequired = countAllRequired + 1;
        currentAttributeId = element.id.replace("RequiredFieldValidator", "");
        select = select + currentAttributeId + ",";
    });

    alert(select);
}

function SetOnChange(attribute, handler, doInit, currentFrame) {
    currentFrame.find("#" + attribute).first().change(handler);

    if (doInit) {
        handler.call();
    }
}

function SetOnInput(attribute, handler, currentFrame) {
    currentFrame.find("#" + attribute).first().on("input", handler);
}

function SetOnChangeOrInput(attribute, handler, doInit, currentFrame) {
    SetOnChange(attribute, handler, doInit, currentFrame);
    SetOnInput(attribute, handler, currentFrame);
}

function SetOnFocus(attribute, handler, currentFrame) {
    currentFrame.find("#" + attribute).first().on("focus", handler);
}

function OnChangeOrInput_numbers(attribute, maxLength, currentFrame) {
    var control = currentFrame.find("#" + attribute).first();
    var oldValue = control.val();
    var newValue = oldValue.substring(0, maxLength).replace(/\D/g, '');//only digits and max length 4
    control.val(newValue);
    return newValue;
}

function OnChangeOrInput_phoneNumber(attribute, currentFrame) {
    var control = currentFrame.find("#" + attribute).first();
    var controlValue = control.val();

    if (controlValue.length === 0) {
        return;
    }

    control.val("+".concat(controlValue.replace(/\D/g, '').substring(0, 16)));////first character must be a + and then only digits and max length 16
}


//scroll to the label corresponding to the error message
function ScrollToElementPosition(attribute) {
    if (attribute) {
        var label = document.activeElement.contentWindow.document.getElementById(
            (typeof attribute === 'string' || attribute instanceof String) ? attribute : attribute.id + "_label");
        var pos = label.getBoundingClientRect();

        window.scrollTo(pos.left, pos.top);
    }
}

function SetValidatorForAttribute(attribute, evaluateFunction, messageSnippet, currentFrame, currentValidators, validatorPrefix = "RequiredFieldValidator", isLink = true) {

    //RemoveValidatorForAttribute(attribute, currentFrame, currentValidators, validatorPrefix);//by default we remove validator in case it already exists to avoid add it twice.

    if (currentValidators === null || currentValidators === undefined) {
        currentValidators = Page_Validators;
    }

    if (typeof (currentValidators) == 'undefined') return;
    if (messageSnippet === null) {
        messageSnippet = '"' + currentFrame.find("#" + attribute + "_label").first().text() + '" ' + (window.parent.location.pathname.includes("/fr-FR/")?"est obligatoire":"is required");
    }

    currentFrame.find("#" + attribute + "_label").first().parent().addClass("required");
    // Create new validator or overwrite existing one on this attribute
    var newValidator = currentFrame.find("#" + validatorPrefix + attribute).get(0);
    if (newValidator === undefined || newValidator === undefined) {
        newValidator = document.createElement('span');
    }
    newValidator.style.display = "none";
    newValidator.id = validatorPrefix + attribute;
    newValidator.controltovalidate = attribute;
    if (isLink) {
        newValidator.errormessage = "<a href='#" + attribute + "_label' onclick='window.parent.ScrollToElementPosition(" + attribute + ");'>" + messageSnippet + "</a>";
    }
    else {
        newValidator.errormessage = "<span>" + messageSnippet + "</span>";
    }

    newValidator.validationGroup = ""; // Set this if you have set ValidationGroup on the form
    newValidator.initialvalue = "";
    if (evaluateFunction !== null) {
        newValidator.evaluationfunction = evaluateFunction;
    }
    else {
        newValidator.evaluationfunction = function () {
            //check if any value (works only for text, number field)
            var control = currentFrame.find("#" + attribute);
            var value = null;

            if (control.hasClass("vertical") || control.hasClass("horizontal")) {//ces classes correspondent à des radiobutton
                value = GetRadioButtonValue(attribute, currentFrame);
                if (isNaN(value)) {
                    return false;
                }
                else {
                    return true;
                }
            }
            else {
                value = control.val();
                if (value === null || value === "") {
                    return false;
                } else {
                    return true;
                }
            }
        }
    }

    // Add the new validator to the page validators array:
    currentValidators.push(newValidator);

    // Wire-up the click event handler of the validation summary link
    currentFrame.find("a[href='#" + attribute + "_label']").on("click", function () { window.parent.ScrollToElementPosition(attribute); });
}

function RemoveValidatorForAttribute(attribute, currentFrame, currentValidators, validatorPrefix = "RequiredFieldValidator") {
    if (currentValidators === null || currentValidators === undefined) {
        currentValidators = Page_Validators;
    }

    $.each(currentValidators, function (index, currentValidator) {
        if (currentValidator !== undefined && currentValidator.id == validatorPrefix + attribute) {
            currentValidators.splice(index, 1);
        }
    });

    currentFrame.find("#" + attribute + "_label").parent().removeClass("required");
}

function OnlyUnique(value, index, self) {
    return self.indexOf(value) === index;
}

function CreateFakePicklist(name, label, options) {
    var picklistContent =
        `<div class="info required">
                <label for="${name}" id="${name}_label" class="field-label">${label}</label>
                </div>
                <div class="control">
                <select name="${name}_name" id="${name}" class="form-control picklist">
					<option value="" label=" " aria-label="Vide"> </option>
					${options.join('')}
				</select></div>`;
    var td1 = document.createElement("td");
    td1.setAttribute("colspan", "1");
    td1.setAttribute("rowspan", "1");
    td1.setAttribute("class", "clearfix cell picklist-cell");
    td1.innerHTML = picklistContent.trim();
    var td2 = document.createElement("td");
    td2.setAttribute("class", "cell zero-cell");
    var tr = document.createElement("tr");
    tr.appendChild(td1);
    tr.appendChild(td2);

    return tr;
}

function RemoveAllOptionsPicklist(selectElement) {
    var i, L = selectElement.options.length - 1;
    for (i = L; i >= 0; i--) {
        selectElement.remove(i);
    }
}



function ControlAvsNumber(pNumAvs) {
    //[0-9]{3}\.?[0-9]{2}\.?[0-9]{3}\.?[0-9]{3} (old) or 756\.?[0-9]{4}\.?[0-9]{4}\.?[0-9]{2}
    var numAvs = pNumAvs;
    var avsRegex = /^756\.?\d{4}\.?\d{4}\.?\d{2}$/; // format: 999.9999.9999.99
    var isOk = false;
    var ii;
    var sum = 0;
    var correctorCode;

    if (numAvs.match(avsRegex)) {
        // check the error detection code
        numAvs = '*' + numAvs.replace(/\./g, '');  // remove dots... beware that it is a regexp expression !!
        for (ii = 1; ii <= 12; ii++) {
            // odd digits are weighted by 1, even digits are weighted with 3
            sum += parseInt(numAvs[ii]) * (ii % 2 == 1 ? 1 : 3);
        }
        correctorCode = (10 - (sum % 10)) % 10;
        if (numAvs[numAvs.length - 1] == correctorCode.toString()) {
            // last char of numAvs is the same as the calculated correctorCode char
            isOk = true;
        }
    }

    return isOk;
}

function OnFocusAvsNumber() {
    var avs = $("#uge_avsnumber").val();
    if (avs === "") {
        $("#uge_avsnumber").val("756.");
    }
}

function ControlRegistrationNumber(matricule) {
    matricule = matricule.replace(/\D/g, '').substring(0, 8);
    if (matricule.length === 8) {
        let total = parseInt(matricule.substring(1, 2)) + parseInt(matricule.substring(3, 4)) + parseInt(matricule.substring(5, 6));
        let doubl = 2 * parseInt(matricule.substring(6, 7));
        if (doubl > 9) {
            total = total + (doubl % 10) + 1
        }
        else {
            total = total + doubl;
        }

        doubl = 2 * parseInt(matricule.substring(4, 5));
        if (doubl > 9) {
            total = total + (doubl % 10) + 1
        }
        else {
            total = total + doubl;
        }

        doubl = 2 * parseInt(matricule.substring(2, 3));
        if (doubl > 9) {
            total = total + (doubl % 10) + 1
        }
        else {
            total = total + doubl;
        }

        doubl = 2 * parseInt(matricule.substring(0, 1));
        if (doubl > 9) {
            total = total + (doubl % 10) + 1
        }
        else {
            total = total + doubl;
        }

        let result = 10 - (total % 10);
        if (result === 10) {
            result = 0;
        }

        return (result === parseInt(matricule.substring(7, 8)));
    }
    else {
        return false;
    }

}


function HandleResidenceSecondaryStudies(currentFrame, attributeLocationName, Page_Validators, messageSnippetCountry, messageSnippetCanton, headerCanton, headerTown) {
    SetValidatorForAttribute("uge_grad_legal_cnt_res_id", null, null, currentFrame, Page_Validators);
    ConvertLookupToDropdown("ref_location", "ref_locations", "uge_grad_legal_cnt_res_id", attributeLocationName, currentFrame, "&$filter=ref_location_type_code eq 910110000 and statecode eq 0&$orderby=ref_srt asc," + attributeLocationName + " asc");
    SetOnChange("uge_grad_legal_cnt_res_id", function () { OnChange_uge_grad_legal_cnt_res_id(currentFrame, Page_Validators); }, true, currentFrame);
}

function OnChange_uge_grad_legal_cnt_res_id(currentFrame, Page_Validators) {
    var city = "uge_grad_legal_cnt_city_id";
    //exception : show buttons for lookup
    currentFrame.find("#" + city).closest("tr").find(".launchentitylookup").show();//could need the same for clearlookupfield
    var countryId = currentFrame.find("#uge_grad_legal_cnt_res_id").val();
    if (countryId === "ce8f0c50-b1a6-ec11-9840-0022486171b3") {//switzerland
        SetValidatorForAttribute(city, null, null, currentFrame, Page_Validators);
        SetFieldDisplay(city, "show", currentFrame);

    }
    else {
        RemoveValidatorForAttribute(city, currentFrame, Page_Validators);
        SetFieldDisplay(city, "hide", currentFrame);//erase true
    }
}

//get reference year for the secondary studies.
function GetYearToFilterSchoolDiploma(currentFrame, iframe = "") {
    if (currentFrame === null && iframe !== "") {
        currentFrame = GetCurrentFrame(iframe);
    }

    var isPassed = GetRadioButtonValue("uge_grad_status_code", currentFrame) === 287030000;
    var yearStr = currentFrame.find("#uge_grad_year").val();
    var year = 1900;
    if (isPassed) {//if Obtenu then we take the year of obtention.
        if (yearStr.length === 4) {
            year = parseInt(yearStr);
        }
    }
    else {//if in progress then we take current year
        year = new Date().getFullYear();
    }

    return year;
}

// clear fields related to grad_school_diploma
function ClearSchoolDiploma(currentFrame) {
    currentFrame.find("#uge_grad_school_diploma_id").val(null);
    currentFrame.find("#uge_grad_school_diploma_id").trigger("change");
    currentFrame.find("#lookupDropdown_uge_grad_school_diploma_id").val("");
}

//a function to check date range in date picker
function CheckDateRange(dateAttribute, currentFrame, minDate, maxDate) {
    var isValid = true;
    var currentDateValue = currentFrame.find("#" + dateAttribute).val();
    var currentDate = new Date(currentDateValue);

    if (minDate > currentDate) {
        isValid = false;
    }

    if (currentDate > maxDate) {
        isValid = false;
    }

    return isValid;
}

//a function to get values stored by liquid in the hidden section of application main menu (web template).
function GetLiquidValue(attribute) {
    return $("#liquid_" + attribute).text();
}


//Etudes secondaires : Réclamer la moyenne si Obtention : encours 
function OnChange_uge_grad_status_code_SetScore(currentFrame, Page_Validators, required, messageSnippet) {
    var uge_grad_score = "uge_grad_grade";
    var uge_grad_score_base = "uge_grad_grade_base";
    var prefix = "RequiredScore";

    var uge_grad_status_code = window.parent.GetDropdownValue("uge_grad_status_code", currentFrame);
    switch (uge_grad_status_code) {
        case 287030000://obtenu 
            SetFieldDisplay(uge_grad_score, "show", currentFrame);
            SetFieldDisplay(uge_grad_score_base, "show", currentFrame);
            if (required) {
                SetValidatorForAttribute(uge_grad_score, null, null, currentFrame, Page_Validators, prefix);
                SetValidatorForAttribute(uge_grad_score_base, null, null, currentFrame, Page_Validators, prefix);
            }
            break;
        case 287030001://en cours
            SetFieldDisplay(uge_grad_score, "hide", currentFrame);//erase true
            SetFieldDisplay(uge_grad_score_base, "hide", currentFrame);//erase true
            if (required) {
                RemoveValidatorForAttribute(uge_grad_score, currentFrame, Page_Validators, prefix);
                RemoveValidatorForAttribute(uge_grad_score_base, currentFrame, Page_Validators, prefix);
            }
            break;
        default: break;
    }
}


//Secondary Studies France and others
//when uge_grad_school_other_b (other school) is updated, handle the display and required fields
function OnChange_uge_grad_school_other_b(currentFrame, Page_Validators, requiredSnippet) {
    var uge_grad_school_diploma_id = "uge_grad_school_diploma_id";
    //var uge_grad_school_name = "uge_grad_school_name";
    var uge_grad_name = "uge_grad_name";

    RemoveValidatorForAttribute(uge_grad_school_diploma_id, currentFrame, Page_Validators);
    RemoveValidatorForAttribute(uge_grad_name, currentFrame, Page_Validators);

    //US3669-----START
    if (currentFrame.find("#uge_grad_school_other_b[US3669]").length > 0) {
        currentFrame.find("#uge_grad_school_other_b").prop("checked", "true");//in scenario US3669 the checkbox is always checked.
    }
    //US3669-----END

    var choiceOther = currentFrame.find("#uge_grad_school_other_b").is(":checked");


    if (choiceOther) {
        SetValidatorForAttribute(uge_grad_name, null, null, currentFrame, Page_Validators);
        SetFieldDisplay(uge_grad_name, "show", currentFrame);
        ClearSchoolDiploma(currentFrame);
        currentFrame.find("#lookupDropdown_uge_grad_school_diploma_id").prop("disabled", true);// we lock the field
        currentFrame.find("#uge_grad_school_diploma_id").prop("disabled", true);// we lock the field
    }
    else {
        SetFieldDisplay(uge_grad_name, "hide", currentFrame);//erase true
        if (!currentFrame.find("#uge_grad_school_diploma_id").closest("fieldset").prop("disabled")) {
            //si le formulaire n'est pas en lecture seule on rend obligatoire
            SetValidatorForAttribute(uge_grad_school_diploma_id, null, null, currentFrame, Page_Validators);
        }
        currentFrame.find("#lookupDropdown_uge_grad_school_diploma_id").prop("disabled", false);// we remove lock on the field
        currentFrame.find("#uge_grad_school_diploma_id").prop("disabled", false);// we remove lock on the field
    }
}

//Secondary Studies France and others
//make sure options in the dropdown of school diploma take the grad year into account
function SetOptionDiploma(item, selectElement, selectedValue, currentFrame) {
    var year = GetYearToFilterSchoolDiploma(currentFrame);
    var start_year = item.ref_start_date === null ? 0 : parseInt(item.ref_start_date.substring(0, 4));
    var end_year = item.ref_end_date === null ? 10000 : parseInt(item.ref_end_date.substring(0, 4));
    if (start_year <= year && end_year >= year) {
        var optionElement = document.createElement("option");
        optionElement.setAttribute("value", item.ref_school_diplomaid);
        optionElement.innerHTML = item.ref_diploma_id.ref_name;

    //US3669-----START
        if (item.ref_diploma_id.ref_name === "Autre/Other")//US3669
        {
            var otherDiplomaCheckbox = currentFrame.find("#uge_grad_school_other_b").first();
            otherDiplomaCheckbox.attr("US3669", "true");//this to recognize that we enter special behavior of US3669.
            selectElement.setAttribute("disabled", "true");
            if (otherDiplomaCheckbox.is(":checked")) {
                otherDiplomaCheckbox.change();//start change event to update behavior surrounding checkbox
            }
            else {
                otherDiplomaCheckbox.click();//set is dirty and trigger change
            }
            //otherDiplomaCheckbox.prop("checked", true);
            SetFieldInstruction("uge_grad_school_other_b", ` `, currentFrame);//we remove instruction above checkbox
        }
    //US3669-----END

        if (item.ref_school_diplomaid === selectedValue) {
            optionElement.setAttribute("selected", "selected");
        }
        selectElement.appendChild(optionElement);
    }
}

function OnChange_uge_grad_score(currentFrame, snippetMessage, fieldName = "uge_grad_grade") {
    var score_base = parseInt(currentFrame.find("#" + fieldName + "_base").val(), 10);
    var score = parseFloat(currentFrame.find("#" + fieldName).val(), 10);

    if (!isNaN(score_base) && !isNaN(score)) {
        if (score > score_base) {
            window.parent.SetFieldAlert(fieldName, snippetMessage, currentFrame);
            currentFrame.find("#" + fieldName).val(null);
        }
        else {
            window.parent.RemoveFieldAlert(fieldName, currentFrame);
        }
    }
}

function CreateRedirectToSubmitButton(currentFrame, buttonLabel) {
    if (!currentFrame.find("#redirectSubmitApplicationButton").length) {
        var redirectSubmitApplicationButton = document.createElement("input");
        redirectSubmitApplicationButton.setAttribute("id", "redirectSubmitApplicationButton");
        redirectSubmitApplicationButton.setAttribute("type", "button");
        redirectSubmitApplicationButton.setAttribute("class", "btn btn-primary button");
        redirectSubmitApplicationButton.setAttribute("value", buttonLabel);
        redirectSubmitApplicationButton.setAttribute("onclick", "javascript:window.parent.RedirectToSubmitApplication();");//ouvre le tab de soumission
        $(redirectSubmitApplicationButton).appendTo(currentFrame.find("div.form-action-container-right"));//append  button next to command bar button.
    }
    else {
        currentFrame.find("#redirectSubmitApplicationButton").show();
    }
}

function RedirectToSubmitApplication() {
    var submitTab = $("#SubmitApplication-tab");
    submitTab.parent().removeClass("disabled");
    submitTab.tab('show');
}

function RedirectToTab(targetTab) {
    $('#' + targetTab).tab('show');
}

//*******************************handle Documents
function RefreshDocuments(applicationId, OnSuccessCallBack, OnErrorCallBack) {
    // IMPORTANT NOTE! please read the following documentation regarding Portals Web API:
    // https://learn.microsoft.com/en-us/power-apps/maker/portals/web-api-overview
    // "webapi.safeAjax" wrapper is based on the code from this page:
    // https://learn.microsoft.com/en-us/power-apps/maker/portals/web-api-http-requests-handle-errors

    // NOTE: The following warnings may not be correct
    // WARNING: Portals doesn't support Formatted Values

    var record = {};
    record.uge_generate_documents = Math.floor(Math.random() * 9999); // Whole Number

    window.parent.webapi.safeAjax({
        type: "PATCH",
        contentType: "application/json",
        url: "/_api/uge_applications(" + applicationId + ")",
        data: JSON.stringify(record),
        success: function (data, textStatus, xhr) {
            // Reload the subgrid of documents
            if (typeof OnSuccessCallBack !== undefined) {
                OnSuccessCallBack();
            }
        },
        error: function (xhr, textStatus, errorThrown) {
            // Manage Error
            if (typeof OnErrorCallBack !== undefined) {
                OnErrorCallBack();
            }
            console.log(errorThrown);
        }
    });
}
//*************************************************


//university studies (common)

function OnChange_uge_grad_status_code(currentFrame, pValidators, isRequiredMessage, infoMessage) {
    var uge_grad_year = "uge_grad_year";
    var uge_grad_expected_month_code = "uge_grad_expected_month_code";
    var uge_grad_status_code = window.parent.GetRadioButtonValue("uge_grad_status_code", currentFrame);//todo : change this to apply 
    if (isNaN(uge_grad_status_code) || uge_grad_status_code === -1) {
        uge_grad_status_code = window.parent.GetDropdownValue("uge_grad_status_code", currentFrame);
    }

    switch (uge_grad_status_code) {
        case 287030000://obtenu
            //if obtenu then set required year of obtention
            window.parent.SetValidatorForAttribute(uge_grad_year, null, null, currentFrame, pValidators);
            window.parent.SetFieldDisplay(uge_grad_year, "show", currentFrame);
            // hide month for obtention
            window.parent.RemoveValidatorForAttribute(uge_grad_expected_month_code, currentFrame, pValidators);
            window.parent.SetFieldDisplay(uge_grad_expected_month_code, "hide", currentFrame);
            break;
        case 287030001://in progress
            //show and required month for obtention
            window.parent.SetValidatorForAttribute(uge_grad_expected_month_code, null, null, currentFrame, pValidators);
            window.parent.SetFieldDisplay(uge_grad_expected_month_code, "show", currentFrame);
            //hide year of obtention
            window.parent.RemoveValidatorForAttribute(uge_grad_year, currentFrame, pValidators);
            window.parent.SetFieldDisplay(uge_grad_year, "hide", currentFrame);
            break;
        default:
            //hide year of obtention
            window.parent.RemoveValidatorForAttribute(uge_grad_year, currentFrame, pValidators);
            window.parent.SetFieldDisplay(uge_grad_year, "hide", currentFrame);
            // hide month for obtention
            window.parent.RemoveValidatorForAttribute(uge_grad_expected_month_code, currentFrame, pValidators);
            window.parent.SetFieldDisplay(uge_grad_expected_month_code, "hide", currentFrame);
            break;
    }
}

//*************************************************

function ConvertDateToUTC(date, addDays = 0) {
    var dateUTC = new Date(date.getUTCFullYear(), date.getUTCMonth(), date.getUTCDate(), date.getUTCHours(), date.getUTCMinutes(), date.getUTCSeconds());
    return dateUTC.setDate(dateUTC.getDate() + addDays);
}

function ResizeIFrameToFitContent(iFrameId) {
    var iFrame = document.getElementById(iFrameId);
    iFrame.width = iFrame.contentWindow.document.body.scrollWidth;
    iFrame.height = iFrame.contentWindow.document.body.scrollHeight;
}

//********************************************** Make a form readonly.
function MakeReadOnlyEntityForm(currentFrame, readonlyMessage, withFieldsetReadOnly = true) {
    if (withFieldsetReadOnly) {
        currentFrame.find("fieldset").prop('disabled', true);
    }
    else {
        currentFrame.find("input").prop('disabled', true);
        currentFrame.find("select").prop('disabled', true);
        currentFrame.find("a.create-action").prop('disabled', true);
    }
    var readonlyInstructionDiv = document.createElement("div");
    readonlyInstructionDiv.setAttribute("class", "alert alert-warning");
    readonlyInstructionDiv.setAttribute("role", "alert");
    readonlyInstructionDiv.innerHTML = readonlyMessage;
    currentFrame.find('fieldset').first().parent().prepend(readonlyInstructionDiv);
    currentFrame.find("#UpdateButton").remove();
}

//********************************************** Add a continue button (chemin de fer)
function AddContinueButton(currentFrame, continueLabel) {
    var isNotFinalStage = $('#wizardNav > li.active').first().next('li').length > 0;
    if (isNotFinalStage) {
        currentFrame.find("#ContinueButton").remove();//bug5999 eviter les boutons Continuer surnuméraires
        var readOnlyNext = `var mainFrame = window.parent.GetCurrentFrame();
    var nextMenu = mainFrame.find('#wizardNav > li.active').first().next('li');
    nextMenu.removeClass('disabled');
    nextMenu.find('a:first').tab('show');`;
        currentFrame.find('#EntityFormPanel').first().append(`<div class="actions">
            <input type="button" value="${continueLabel}" onclick="javascript:${readOnlyNext}" id="ContinueButton" class="btn btn-primary form-action-container-right">
            </div>`);
    }
}

//********************************************** Build a control to ease the selection of a university in a country  //US2256 (AQU)
function FindUniversityByCountry(currentFrame, countryIdAttribute, universityIdAttribute, cityLabel, universityLabel) {
    //alert("FindUniversityByCountry : countryId = " + currentFrame.find("#" + countryIdAttribute).val());
    let countryId = currentFrame.find("#" + countryIdAttribute).val()
    if (countryId !== undefined && countryId !== null && countryId !== "") {

        let idCity = "uge_university_city_code";
        let idUniversity = "uge_university_code";

        var urlLookupDropdown = "/_api/uge_universities?$select=uge_universityid,uge_name,_uge_country_id_value,uge_city&$filter=statecode eq 0 and _uge_country_id_value eq " + countryId + "&$orderby=uge_city asc,uge_name asc";
        webapi.safeAjax({
            type: "GET",
            url: urlLookupDropdown,
            contentType: "application/json",
            success: function (universities, status, xhr) {
                var universityValue = InitUniversityValue(universities, currentFrame, universityIdAttribute);

                let labelCity = cityLabel;
                let optionsCity = GetCityOptions(currentFrame, universities, countryId);
                var trCity = CreateFakePicklist(idCity, labelCity, optionsCity);
                var selectCity = trCity.querySelector('#' + idCity);
                $(selectCity).val(universityValue.uge_city);

                let labelUniversity = universityLabel;
                let optionsUniversity = GetUniversityOptions(currentFrame, universities, countryId, universityValue.uge_city);
                var trUniversity = CreateFakePicklist(idUniversity, labelUniversity, optionsUniversity);
                var selectUniversity = trUniversity.querySelector('#' + idUniversity);
                $(selectUniversity).val(universityValue.uge_universityid);

                //onchange
                selectCity.addEventListener("change", function () {
                    RemoveAllOptionsPicklist(selectUniversity);
                    optionsUniversity = GetUniversityOptions(currentFrame, universities, countryId);
                    selectUniversity.innerHTML = '<option selected="selected" value="" label=" " aria-label="Vide"> </option>' + optionsUniversity.join('');
                });

                selectUniversity.addEventListener("change", function () {
                    //alert("changeProgram = " + selectProgram.value);
                    let lookupIdElement = currentFrame.find("#" + universityIdAttribute);
                    lookupIdElement.val(selectUniversity.value);
                    lookupIdElement.addClass("dirty");
                    currentFrame.find("#" + universityIdAttribute + "_entityname").val("uge_university");
                });

                //insert picklists
                currentFrame.find("#" + idCity).closest("tr").remove();//we remove previous dropdown if existing
                currentFrame.find("#" + idUniversity).closest("tr").remove();//we remove previous dropdown if existing
                let trSibling = currentFrame.find("#" + universityIdAttribute).closest("tr").get(0);
                let parentDiv = trSibling.parentNode;
                parentDiv.insertBefore(trCity, trSibling);
                parentDiv.insertBefore(trUniversity, trSibling);

                trSibling.style.display = 'none';//hide the lookup
            }
        });
    }
}
function GetCityOptions(currentFrame, universities, countryId) {
    return universities.value.filter(function (v) {
        return v._uge_country_id_value === countryId //only universities in this country
    }).map(function (v) { return "<option value='" + v.uge_city + "'>" + v.uge_city + "</option>"; }).filter(OnlyUnique);
}

function GetUniversityOptions(currentFrame, universities, countryId, cityId = null) {
    if (cityId === null) {
        cityId = currentFrame.find("#uge_university_city_code").val();
    }
    return universities.value.filter(function (v) { return v._uge_country_id_value === countryId && v.uge_city === cityId; }).map(function (v) { return "<option value='" + v.uge_universityid + "'>" + v.uge_name + "</option>"; }).filter(OnlyUnique);
}

function InitUniversityValue(universities, currentFrame, lookupAttribute) {
    var lookupAttributeValue = currentFrame.find("#" + lookupAttribute).val();
    //uge_universityid,uge_name,_uge_country_id_value,uge_city
    var universityValue = {
        uge_universityid: "",
        uge_name: "",
        uge_city: ""
    };

    if (lookupAttributeValue !== undefined && lookupAttributeValue !== null && lookupAttributeValue !== "") {
        var universityValueResult = universities.value.filter(function (v) {
            return v.uge_universityid === lookupAttributeValue;
        });
        if (universityValueResult.length > 0) {
            universityValue = universityValueResult[0];
        }
    }

    return universityValue
}
//***************************************************************************************

//US2404 information fields to display TASK2668
function GetInformationFields(currentFrame, infoFields, labelFields, targetToAppendTo, insideTable = true) {
    infoFields.forEach(function (value, index) {
        var elementValue = GetLiquidValue(value).replace(/(?:\r\n|\r|\n)/g, '<br>');;//info must be existing in liquid section, + we add html line breaks when necessary
        var elementLabel = labelFields[index];
        var innerHtml = `<div class="info"><label for="${value}" id="${value}_label" class="field-label" readonly="readonly">${elementLabel}</label>
<br></div><div class='control'><div id="${value} class='description above' style='text-align:justify;'">${elementValue}</div></div>`;

        currentFrame.find("#" + value).remove();//delete and replace if existing
        if (elementValue.length > 0) {

            if (insideTable) {
                var td1 = document.createElement("td");
                td1.setAttribute("colspan", "1");
                td1.setAttribute("rowspan", "1");
                td1.setAttribute("class", "clearfix cell text");
                td1.innerHTML = innerHtml;
                var td2 = document.createElement("td");
                td2.setAttribute("class", "cell zero-cell");
                var tr = document.createElement("tr");
                tr.appendChild(td1);
                tr.appendChild(td2);

                currentFrame.find(targetToAppendTo).append(tr);
            }
            else {
                currentFrame.find(targetToAppendTo).append(innerHtml);
            }
        }
    });
}

//BUG2730 this function scroll to top position with a padding of "positionPadding" pixel within timeAnimation milliseconds.
function ScrollToTopWizardNavigation(positionPadding = 0, timeAnimation = 1000) {
    $("html, body").animate({ scrollTop: positionPadding }, timeAnimation);
}

//reload iframe on wizard
function LoadIframeOnWizard(iframeId) {
    var iframeElement = document.getElementById(iframeId);
    iframeElement.src = iframeElement.getAttribute("src-back");
}

//***************************************************************************************
// Functions concerning AVS number verifications
//***************************************************************************************

//At each focus on the avs number control avs number validity and check format
function OnFocus_uge_avs_nbr(currentFrame) {
    var control = currentFrame.find("#uge_avs_nbr").first();
    var avsValue = control.val();
    var isOk = ControlAvsNumber(avsValue);
    if (!isOk || avsValue.length === 0) {
        //control.val("756.");
        control.val(null);
        RemoveFieldAlert("uge_avs_nbr", currentFrame);
    }
    else if (avsValue.length == 13) {
        control.val(avsValue.substring(0, 3) + "." + avsValue.substring(3, 7) + "." + avsValue.substring(7, 11) + "." + avsValue.substring(11, 13));
    }
}

//At each input on the avs number the function adapt the display
function OnInput_uge_avs_nbr(currentFrame) {
    var avs = currentFrame.find("#uge_avs_nbr").first();
    var avsValue = avs.val();
    if (avsValue.length !== 0) {
        avsValue = avsValue.substring(0, avsValue.length - 1) + avsValue[avsValue.length - 1].replace(/\D/g, '');//only digits
        if ([3, 8, 13].indexOf(avsValue.length) >= 0) { avsValue = avsValue + "."; }// add dot when necessary
        avs.val(avsValue.substring(0, 16));//max length 16/
    }
}

//control the avs number before submit
function ValidateAvsBeforeSubmit(currentFrame, messageSnippet) {
    if (typeof (Page_Validators) == 'undefined') return;
    // Create new validator
    var newValidator = document.createElement('span');
    newValidator.style.display = "none";
    newValidator.id = "uge_avs_nbrValidator";
    newValidator.controltovalidate = "uge_avs_nbr";
    newValidator.errormessage = "<a href='#uge_avs_nbr_label'>" + messageSnippet + "</a>";
    newValidator.validationGroup = ""; // Set this if you have set ValidationGroup on the form
    newValidator.initialvalue = "";
    newValidator.evaluationfunction = function () {
        var value = currentFrame.find("#uge_avs_nbr").val();
        if (value == null || value == "") {
            return true;
        } else {
            return ControlAvsNumber(value);
        }
    };

    // Add the new validator to the page validators array:
    Page_Validators.push(newValidator);

    // Wire-up the click event handler of the validation summary link
    $("a[href='#uge_avs_nbr_label']").on("click", function () { scrollToAndFocus('uge_avs_nbr_label', 'uge_avs_nbr'); });
}

//on changing the avs Number the function control the validity of the avs number
function OnChange_uge_avs_nbr(currentFrame, messageSnippet) {
    var avsValue = currentFrame.find("#uge_avs_nbr").first().val();
    var isOk = ControlAvsNumber(avsValue);
    if (!isOk && avsValue.length > 0) {
        SetFieldAlert("uge_avs_nbr", messageSnippet, currentFrame)
    }
    else {
        RemoveFieldAlert("uge_avs_nbr", currentFrame);
    }
}

function SetSpaceAboveField(attribute, currentFrame, htmlSpace = '<br>') {
    SetHtmlAboveField(attribute, htmlSpace, currentFrame);
}

function SetHtmlAboveField(attribute, html, currentFrame) {
    $(html).insertBefore(currentFrame.find("#" + attribute).first().closest("tr"));
}

function SetEmptyLabelInDropDown(attribute, currentFrame, labelFR = "-- ne s'applique pas", labelEN = "-- not applicable") {
    var emptyOption = currentFrame.find("#" + attribute).children("option").first();
    var emptyLabel = window.parent.location.pathname.includes("/fr-FR/") ? labelFR : labelEN;
    if (currentFrame.find("#" + attribute + ` > option[label="${emptyLabel}"]`).length === 0) {
        $(`<option value="" label="${emptyLabel}" aria-label="Blank">${emptyLabel}`).insertAfter(emptyOption);
    }
}

function ConfirmReadStage(tableSet, recordId, attributeName) {
    if (attributeName.includes("uge_ok_")) {
        var record = {};
        record[attributeName] = 287030001; // Choice

        webapi.safeAjax({
            type: "PATCH",
            contentType: "application/json",
            url: `/_api/${tableSet}(${recordId})`,
            data: JSON.stringify(record),
            success: function (data, textStatus, xhr) {
                console.log(attributeName +" Record updated");
            },
            error: function (xhr, textStatus, errorThrown) {
                console.log(xhr);
            }
        });
    }
}

function OnChange_uge_is_care_of_code(currentFrame, validators) {
    var uge_is_care_of_code = GetRadioButtonValue("uge_is_care_of_code", currentFrame);
    var uge_address1_line1 = "uge_address1_line1";
    switch (uge_is_care_of_code) {
        case 287030000:
            //if NO 
            SetFieldDisplay(uge_address1_line1, "show", currentFrame);
            SetValidatorForAttribute(uge_address1_line1, null, null, currentFrame, validators);

            break;
        case 287030001:
        //if YES
        default:
            SetFieldDisplay(uge_address1_line1, "hide", currentFrame);//erase true
            RemoveValidatorForAttribute(uge_address1_line1, currentFrame, validators);
            break;
    }
}
